// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef VEC_OF_INTSETS_PAIRS_H__RTG
#define VEC_OF_INTSETS_PAIRS_H__RTG

#include <boost/unordered_set.hpp>
#include "pairs.h"

namespace RTG
{
    /** \brief A class for storing the already reported pairs,
            which uses a vector of unordered-sets (hash maps).
	**/
    /*! The data structure is a vector of unordered-sets.
        The i-th entry of the vector stores the set of reported neighbors of the i-th data point.
    */
	class VecOfIntSetsPairsStruct : public PairsStruct
    {
    public:
		typedef boost::unordered_set<int>       Int_set;
		typedef std::vector<Int_set>            Pairs;
		
        /** \brief Constructor */
        VecOfIntSetsPairsStruct(unsigned int n, bool add_once = true) 
			: PairsStruct(n, add_once)
        {
			for (unsigned int i(0); i < n; ++i)
			{
				pairs_.push_back(Int_set());
			}
        }


		/** \brief Check whether a pair exists in the structure */
		virtual bool pair_exists(int i, int j) const
		{
            assert (i != j);
            return (i < j) ? 
                    pairs_[i].find(j) != pairs_[i].end() :
                    pairs_[j].find(i) != pairs_[j].end();
		}
		
		/** \brief Add pair to the structure */
		virtual void add_pair(int i, int j)
		{
            assert (i != j);
            if (add_once_)
            {
                if (i < j) 
                    pairs_[i].insert(j);
                else
                    pairs_[j].insert(i);
            }
            else
            {
                pairs_[i].insert(j);
                pairs_[j].insert(i);
            }
			return;
		}

		/** \brief Clear the datastructure */
        virtual void clear(void)
		{
    		for (unsigned int i(0); i < num_of_points_; ++i)
			{
				pairs_[i].clear();
			}
		}
		
		/** \brief places the indices of the neighbors of index i in the given vector */
		virtual void neighbors_of_i(int i, std::vector<int>& neighborsVec) const
		{
			Int_set::iterator it;
			for (it = pairs_[i].begin(); it != pairs_[i].end(); ++it)
				neighborsVec.push_back(*it);
		}

		/** \brief places the reported pairs of indices in the given vector */
		virtual void all_pairs(std::vector<std::pair<int, int>>& pairsVec) const
		{
            for (std::size_t i(0); i < pairs_.size(); ++i)
			{
				Int_set::iterator it;
				for (it = pairs_[i].begin(); it != pairs_[i].end(); ++it)
					pairsVec.push_back(std::make_pair(i, *it));
			}
		}
	 
	 private:
		/// \brief A data structure holding the reported pairs
		Pairs pairs_;
		
    };

} //namespace RTG



#endif //VEC_OF_INTSETS_PAIRS_H__RTG